#include <TinyGPS++.h>        // include TinyGPS++ library
#include <TimeLib.h>          // include Time library
#include <SoftwareSerial.h>   // include Software serial library
#include <LiquidCrystal_I2C.h>// include LiquidCrystal library

TinyGPSPlus gps;

#define S_RX    4   // define software serial RX pin
#define S_TX    3   // define software serial TX pin

SoftwareSerial SoftSerial(S_RX, S_TX);   // configure SoftSerial library

// initialise the LCD 1602 I2C
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);

#define time_offset   10800  // define a clock offset of 10800 seconds (3 hours) ==> UTC + 3

// variable definitions
char Time[]  = "00:00:00";
char Date[]  = "00-00-2000";
byte last_second, Second, Minute, Hour, Day, Month;
int Year;

void setup(void)
{
  SoftSerial.begin(9600);  // initialize software serial at 9600 baud
  lcd.begin (16, 2); // define the LCD as 16 column by 2 rows
  // switch on the backlight
  lcd.setBacklightPin(3, POSITIVE);
  lcd.setBacklight(HIGH);
  lcd.setCursor(2, 0); // move LCD cursor to column 2, row 0
  lcd.print("ACOPTEX.COM");
  lcd.setCursor(0, 1); // move LCD cursor to column 0, row 1
  lcd.print("GPS CLOCK  UTC+3");
  delay(2000);
  lcd.clear();
}

void loop()
{

  while (SoftSerial.available() > 0)
  {
    if (gps.encode(SoftSerial.read()))
    {
      // get time from GPS module
      if (gps.time.isValid())
      {
        Minute = gps.time.minute();
        Second = gps.time.second();
        Hour   = gps.time.hour();
      }
      // get date drom GPS module
      if (gps.date.isValid())
      {
        Day   = gps.date.day();
        Month = gps.date.month();
        Year  = gps.date.year();
      }

      if (last_second != gps.time.second()) // if time has changed
      {
        last_second = gps.time.second();

        // set current UTC time
        setTime(Hour, Minute, Second, Day, Month, Year);
        // add the offset to get local time
        adjustTime(time_offset);

        // update time array
        Time[0]  = hour()   / 10 + '0';
        Time[1]  = hour()   % 10 + '0';
        Time[3]  = minute() / 10 + '0';
        Time[4] = minute() % 10 + '0';
        Time[6] = second() / 10 + '0';
        Time[7] = second() % 10 + '0';

        // update date array
        Date[0]  =  day()   / 10 + '0';
        Date[1]  =  day()   % 10 + '0';
        Date[3]  =  month() / 10 + '0';
        Date[4] =  month() % 10 + '0';
        Date[8] = (year()  / 10) % 10 + '0';
        Date[9] =  year()  % 10 + '0';
        // print time & date
        print_wday(weekday());   // print day of the week
        lcd.setCursor(0, 1);     // move cursor to column 0 row 1
        lcd.print(Time);         // print time (HH:MM:SS)
        lcd.setCursor(0, 0);     // move cursor to column 0 row 0
        lcd.print(Date);         // print date (DD-MM-YYYY)
        lcd.setCursor(11, 1);     // move cursor to column 11 row 1
        lcd.print("UTC+3");       // print text
      }
    }
  }
}

// function for displaying day of the week
void print_wday(byte wday)
{
  lcd.setCursor(11, 0);  // move cursor to column 11, row 0
  switch (wday)
  {
    case 1:  lcd.print("SUN");   break;
    case 2:  lcd.print("MON");   break;
    case 3:  lcd.print("TUE");   break;
    case 4:  lcd.print("WED");   break;
    case 5:  lcd.print("THU");   break;
    case 6:  lcd.print("FRI");   break;
    default: lcd.print("SAT");
  }
}
